/*
 * This file is part of pygal
 *
 * A python svg graph plotting library
 * Copyright © 2012 Kozea

 * This library is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with pygal. If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * Styles from config
 */

{{ id }}{
  background-color: {{ style.background }};
}

{{ id }}path,
{{ id }}line,
{{ id }}rect,
{{ id }}circle {
  -webkit-transition: {{ style.transition }};
  -moz-transition: {{ style.transition }};
  transition: {{ style.transition }};
}

{{ id }}.graph > .background {
  fill: {{ style.background }};
}

{{ id }}.plot > .background {
  fill: {{ style.plot_background }};
}

{{ id }}.graph {
  fill: {{ style.foreground }};
}

{{ id }}text.no_data {
  fill: {{ style.foreground_strong }};
}

{{ id }}.title {
  fill: {{ style.foreground_strong }};
}

{{ id }}.legends .legend text {
  fill: {{ style.foreground }};
}

{{ id }}.legends .legend:hover text {
  fill: {{ style.foreground_strong }};
}

{{ id }}.axis .line {
  stroke: {{ style.foreground_strong }};
}

{{ id }}.axis .guide.line {
  stroke: {{ style.foreground_subtle }};
}

{{ id }}.axis .major.line {
  stroke: {{ style.foreground }};
}

{{ id }}.axis text.major {
  fill: {{ style.foreground_strong }};
}

{{ id }}.axis.y .guides:hover .guide.line,
{{ id }}.line-graph .axis.x .guides:hover .guide.line,
{{ id }}.stackedline-graph .axis.x .guides:hover .guide.line,
{{ id }}.xy-graph .axis.x .guides:hover .guide.line {
  stroke: {{ style.foreground_strong }};
}

{{ id }}.axis .guides:hover text {
  fill: {{ style.foreground_strong }};
}


{{ id }}.reactive {
  fill-opacity: {{ style.opacity }};
  stroke-opacity: {{ style.stroke_opacity }};
  stroke-width: {{ style.stroke_width }};
}

{{ id }}.ci {
    stroke: {{ style.foreground }};
}

{{ id }}.reactive.active,
{{ id }}.active .reactive {
  fill-opacity: {{ style.opacity_hover }};
  stroke-opacity: {{ style.stroke_opacity_hover }};
  stroke-width: {{ style.stroke_width_hover }};
}

{{ id }}.ci .reactive.active {
  stroke-width: 1.5;
}

{{ id }}.series text {
  fill: {{ style.foreground_strong }};
}

{{ id }}.tooltip rect {
  fill: {{ style.plot_background }};
  stroke: {{ style.foreground_strong }};
  -webkit-transition: opacity {{ style.transition }};
  -moz-transition: opacity {{ style.transition }};
  transition: opacity {{ style.transition }};
}

{{ id }}.tooltip .label {
  fill: {{ style.foreground }};
}

{{ id }}.tooltip .label {
  fill: {{ style.foreground }};
}

{{ id }}.tooltip .legend {
  font-size: .8em;
  fill: {{ style.foreground_subtle }};
}

{{ id }}.tooltip .x_label {
  font-size: .6em;
  fill: {{ style.foreground_strong }};
}

{{ id }}.tooltip .xlink {
  font-size: .5em;
  text-decoration: underline;
}

{{ id }}.tooltip .value {
  font-size: 1.5em;
}

{{ id }}.bound {
  font-size: .5em;
}

{{ id }}.max-value {
  font-size: .75em;
  fill: {{ style.foreground_subtle }};
}

{{ id }}.map-element {
  fill: {{ style.plot_background }};
  stroke: {{ style.foreground_subtle }} !important;
}

{{ id }}.map-element .reactive {
  fill-opacity: inherit;
  stroke-opacity: inherit;
}

{{ colors }}

{{ strokes }}
